<?php
/*
Plugin Name: Prioritize Hooks
Plugin URI: http://jonathanweatherhead.com/projects/prioritize-hooks/
Description: Prioritize Hooks allows the overriding of the priority of various filters and actions hooked by plugins and themes.
Version: 1.0.3
Author: Jonathan Weatherhead
Author URI: http://jonathanweatherhead.com
License: GPL2
*/

/*  Copyright 2012  Jonathan Weatherhead  (email : jon@planetjon.ca)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

include_once(plugin_dir_path(__file__) . 'prioritizehooks-admin.php');

add_action( 'plugins_loaded', array('PrioritizeHooks', 'init') );

class PrioritizeHooks {
	const tag = 'PRIORITIZE_HOOKS';
	const installables_only = false;

	private static $instance;

	private $pluginroot;
	private $themeroot;
	private $options;
	private $defaultpriorities;

	static function init() {
		//standard plugin hooks
		register_activation_hook( __FILE__, array('PrioritizeHooks', 'activate') );
		register_deactivation_hook( __FILE__, array('PrioritizeHooks', 'deactivate') );
		register_uninstall_hook( __FILE__, array('PrioritizeHooks', 'uninstall') );

		//override hooks in 'wp_loaded' hook
		add_action( 'wp_loaded', array(__CLASS__, 'overrideCallbackPriorities') );

		//Set up admin page & menu
		add_action( 'admin_menu', array('PrioritizeHooksAdmin', 'menuSetup') );
		add_action( 'admin_init', array('PrioritizeHooksAdmin', 'hooksOverrideSetup') );

		load_plugin_textdomain('prioritize-hooks', false, dirname( plugin_basename(__FILE__) ) . '/languages/');
	}

	static function activate() {
		add_option( self::tag, array( 'priorities' => array() ) );
	}

	static function deactivate() {}

	static function uninstall() {
		delete_option(self::tag);
	}

	function __construct() {
		$this->pluginroot = strtr( dirname( plugin_dir_path(__FILE__) ), '\\', '/');
		$this->themeroot = strtr(get_theme_root(), '\\', '/');
		$this->options = get_option( self::tag, array( 'priorities' => array( 'priorities' => array() ) ) );
		$this->defaultpriorities = array();
	}

	static function overrideCallbackPriorities() {
		$overrides = apply_filters( 'prioritize_hooks_override_filter_priorities', self::getOverridenCallbacks() );
		$installedcallbacks = apply_filters( 'prioritize_hooks_override_filter_callbacks', self::readHookCallbacks(self::installables_only) );

		foreach($installedcallbacks as $callback) {
			extract($callback, EXTR_PREFIX_ALL, 'cb');
			if( isset($overrides[$cb_tag][$cb_fname]) ) {
				self::getInstance()->defaultpriorities[$cb_tag][$cb_fname] = $cb_priority;
				remove_action($cb_tag, $cb_runnable, $cb_priority, $cb_fargs);
				add_action($cb_tag, $cb_runnable, $overrides[$cb_tag][$cb_fname], $cb_fargs);
			}
		}
	}

	static function getOverridenCallbacks() {
		return self::getInstance()->options['priorities'];
	}

	static function getDefaultPriorities() {
		return self::getInstance()->defaultpriorities;
	}

	static function readHookCallbacks($includecore = false) {
		global $wp_filter;
		$instance = self::getInstance();
		$callbacks = array();

		foreach($wp_filter as $tag=>$priorities)
			foreach($priorities as $priority=>$fhashes)
				foreach($fhashes as $fhash=>$runnable)
					if(! $reflection = self::reflectFunction($runnable['function']) )
						continue;
					elseif($includecore || strpos($reflection['file'], $instance->pluginroot) !== false || strpos($reflection['file'], $instance->themeroot) !== false)
						$callbacks[] = self::buildHookOverride(
							$tag,
							$priority,
							$fhash,
							$reflection['name'],
							$runnable['accepted_args'],
							$runnable['function'],
							$reflection['file']
						);

		return $callbacks;
	}

	static function installedCallbacksByPackage() {
		$callbacks = array();

		foreach(self::readHookCallbacks(self::installables_only) as $callback)
			$callbacks[self::nameOfInstallable($callback['file'])][] = $callback;

		return $callbacks;
	}

	static function buildHookOverride($tag, $priority, $fhash, $fname, $fargs, $runnable, $file) {
		return compact('tag', 'priority', 'fhash', 'fname', 'fargs', 'runnable', 'file');
	}

	private static function getInstance() {
		if(! self::$instance)
			self::$instance = new self;

		return self::$instance;
	}

	private static function nameOfInstallable($path) {
		$instance = self::getInstance();
		if( preg_match("#^({$instance->pluginroot}|{$instance->themeroot})/([^/]+)#", $path, $matches) )
			return $matches[2];
		else
			return false;
	}

	private static function reflectFunction($runnable) {
		$reflection = array();

		try {
			if( is_array($runnable) )
				$reflector = new ReflectionMethod($runnable[0], $runnable[1]);
			else
				$reflector = new ReflectionFunction($runnable);

			$reflection = array_merge($reflection, array(
				'name' => ($reflector instanceof ReflectionMethod
					? "{$reflector->class}::"
					: '')
					. $reflector->getName(),
				'file' => strtr($reflector->getFileName(), '\\', '/')
			));
		}
		catch(Exception $e) {
		}

		return $reflection;
	}

}

?>